/*
 *  MIDI pattern processor
 *  by Shawn Hargreaves, 1995/96
 *
 *  Mutation routines for altering the dynamics of a pattern.
 */


#include "defs.h"


MUTATE_FUNC expression_func;


/* various different types of expression effect */

#define EXPRESSION_QUIETER          1
#define EXPRESSION_LOUDER           2
#define EXPRESSION_FADE_IN          3
#define EXPRESSION_FADE_OUT         4
#define EXPRESSION_CLONE            5


LIST_MEMBER expression_params[] =
{
    /* <description>                <data>                        <coolness> */
    { "Quieter",                    EXPRESSION_QUIETER,           MAX_PROB/4 },
    { "Louder",                     EXPRESSION_LOUDER,            MAX_PROB/4 },
    { "Fade In",                    EXPRESSION_FADE_IN,           MAX_PROB/4 },
    { "Fade Out",                   EXPRESSION_FADE_OUT,          MAX_PROB/4 },
    { "Clone Dynamics",             EXPRESSION_CLONE,             MAX_PROB/4 },
    { NULL, 0, 0 }
};



MUTATOR expression =
{
    expression_func,            /* changes the dynamics of a pattern */
    expression_params           /* what sort of change? */
};



/* expression_func:
 *  Changes the dynamics of a pattern.
 */
PATTERN *expression_func(p1, p2, i)
PATTERN *p1;
PATTERN *p2;
int i;
{
    PATTERN *p;
    int c, c2;
    float mul1, mul2;
    float mul;

    p = clone_pattern(p1);

    if ((p) && (p->length > 1)) {
	if (i == EXPRESSION_CLONE) {
	    /* copy the volume pattern of p2 */ 
	    TRACE("expression_func(EXPRESSION_CLONE)\n");
	    c2 = 0;
	    for (c=0; c<p->length; c++) {
		p->note[c].vel = p2->note[c2].vel;
		c2++;
		if (c2 >= p2->length)
		    c2 = 0;
	    }
	}
	else {
	    /* make simple alteration to p1 volumes */
	    if (i == EXPRESSION_QUIETER) {
		TRACE("expression_func(EXPRESSION_QUIETER)\n");
		mul1 = mul2 = 0.5;
	    }
	    else if (i == EXPRESSION_LOUDER) {
		TRACE("expression_func(EXPRESSION_LOUDER)\n");
		mul1 = mul2 = 1.25;
	    }
	    else if (i == EXPRESSION_FADE_IN) {
		TRACE("expression_func(EXPRESSION_FADE_IN)\n");
		mul1 = 0.5;
		mul2 = 1.25;
	    }
	    else {  /* i must be EXPRESSION_FADE_OUT */
		TRACE("expression_func(EXPRESSION_FADE_OUT)\n");
		mul1 = 1.25;
		mul2 = 0.5;
	    }

	    /* change the volumes */
	    for (c=0; c<p->length; c++) {
		mul = mul1 + ((mul2 - mul1) * (float)c) / (float)(p->length-1);
		p->note[c].vel = (int)((float)p->note[c].vel * mul);
	    }
	}
    }

    return p;
}


