/*
 *  MIDI pattern processor
 *  by Shawn Hargreaves, 1995/96
 *
 *  Simple user interface code.
 */


#include "defs.h"


/* if set, the user interface needs redrawing */
int ui_update = FALSE;


/* if set, the user interface is displaying a probability weighting list */
LIST_MEMBER *view_list = NULL;


/* the message displayed at the top of the screen */
char view_title[80];



/* draw_list:
 *  Displays a list of LIST_MEMBER structures and their weightings.
 */
void draw_list(list, nested)
LIST_MEMBER *list;
int nested;
{
    char buf[80];
    int c;
    char ch = '0';
    MUTATOR *m;

    if (!list)
	return;

    screen_out(view_title);
    screen_out(" (Esc to exit)\r\n\n");

    for (c=0; list[c].data; c++) {
	sprintf(buf, "\r\n(%c/%c)     %s:%*s%-3ld", 'a'+c, 'A'+c, 
	    list[c].desc, 22-(int)strlen(list[c].desc), "", list[c].coolness);

	if (nested) {
	    m = (MUTATOR *)list[c].data;
	    if (m->integer_list) {
		sprintf(buf+strlen(buf), "    (%c to tweak)", ch);
		ch++;
	    }
	}

	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();
    }

    screen_char('\r');

    while (c++ < 18)
	screen_char('\n');
}



/* update_screen:
 *  Redraws the screen display whenever anything changes.
 */
void update_screen()
{
    char buf[80];

    home();

    if (view_list) {
	draw_list(view_list, (view_list == the_mutators));
    }
    else {
	/* keep midi playback going! */
	do_midi();

	screen_out("MIDI Pattern Processor, by Shawn Hargreaves, 1995/98\r\n\n\n");
	screen_out("v:        View mutator probabilities\r\n");
	screen_out("Space:    Pause\r\n");
	screen_out("Del:      Clear patterns\r\n");
	screen_out("Esc:      Quit\r\n");

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(c/C)     MIDI channel:          %d", midi_channel+1);
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(t)       Tick:                  %s", metronome_flag ? "On" : "Off");
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(s/S)     Speed:                 %d", metronome_speed);
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(q)       Quantize:              %s", quantize_flag ? "On" : "Off");
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(r/R)     Quantize ratio:        1/%d", quantize_speed);
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(d/D)     Density:               %d", pattern_density);
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	sprintf(buf, "\r\n(m/M)     Mutation rate:         %d", mutation_rate);
	screen_out(buf);
	del_to_eol();

	/* keep midi playback going! */
	do_midi();

	screen_out("\r\n\n\n\n\n\n");
    }

    if (is_recording())
	screen_out("Recording");
    else
	screen_out("Thru Mode");

    /* keep midi playback going! */
    do_midi();

    screen_out("\r\n\n\n<->                            <0>                           <+>\r\n");
}



/* draw_feedback:
 *  Displays user feedback (pitch bend position).
 */
void draw_feedback(bend)
long bend;
{
    int x;

    bend = (bend / 4) + 32;

    screen_char('\r');

    for (x=0; x<bend; x++)
	screen_char('-');

    screen_char('*');

    while (x++ < 63)
	screen_char('-');

    screen_char('\r');
}



/* ui_init:
 *  Sets up the user interface code at program startup.
 */
void ui_init()
{
    hide_cursor();
    clear_screen();
    update_screen();
}



/* ui_cleanup:
 *  Called at program termination to clean up the user interface code.
 */
void ui_cleanup()
{
    clear_screen();
    show_cursor();
}



/* do_ui:
 *  Called at regular intervals during the program to process any waiting 
 *  user input (keypresses). If it sets *finish to a non-zero value, the
 *  main control loop will terminate the program.
 */
void do_ui(finish)
int *finish;
{
    static int recording = FALSE;
    static long bend = 0xFFFF;

    int c, d, x;
    long t;
    MUTATOR *m;

    while (keypressed()) {
	c = readkey();

	if (view_list) {
	    if ((c=='v') || (c=='V') || (c==27)) {
		if (view_list == the_mutators)
		    view_list = NULL;
		else {
		    view_list = the_mutators;
		    strcpy(view_title, "Mutator Probabilities");
		}
		ui_update = TRUE;
		clear_screen();
		bend = 0xFFFF;
	    }
	    else if ((c >= '0') && (c <= '9') && (view_list == the_mutators)) {
		for (x=0; view_list[x].data; x++) {
		    m = (MUTATOR *)view_list[x].data;
		    if (m->integer_list) {
			c--;
			if (c < '0') {
			    view_list = m->integer_list;
			    sprintf(view_title, "%s types", the_mutators[x].desc);
			    ui_update = TRUE;
			    clear_screen();
			    bend = 0xFFFF;
			    break;
			}
		    }
		}
	    }
	    else {
		if (c >= 'a') {
		    d = -16;
		    c -= 'a';
		}
		else {
		    d = 16;
		    c -= 'A';
		}
		if (c >= 0) {
		    for (x=0; view_list[x].data; x++) {
			if (x==c) {
			    view_list[c].coolness += d;
			    if (view_list[c].coolness < 0)
				view_list[c].coolness = 0;
			    else if (view_list[c].coolness > MAX_PROB)
				view_list[c].coolness = MAX_PROB;
			    ui_update = TRUE;
			    break;
			}
		    }
		}
	    }
	}
	else {
	    switch (c) {

		case 'c':
		    if (midi_channel > 0) {
			change_midi_channel(midi_channel-1);
			ui_update = TRUE;
		    }
		    break;

		case 'C':
		    if (midi_channel < 15) {
			change_midi_channel(midi_channel+1);
			ui_update = TRUE;
		    }
		    break;

		case 't':
		case 'T':
		    metronome_flag = !metronome_flag;
		    ui_update = TRUE;
		    break;

		case 's':
		    if (metronome_speed > 16) {
			metronome_speed--;
			ui_update = TRUE;
		    }
		    break;

		case 'S':
		    if (metronome_speed < 512) {
			metronome_speed++;
			ui_update = TRUE;
		    }
		    break;

		case 'q':
		    quantize_flag = !quantize_flag;
		    ui_update = TRUE;
		    break;

		case 'r':
		    if (quantize_speed > 1) {
			quantize_speed--;
			ui_update = TRUE;
		    }
		    break;

		case 'R':
		    if (quantize_speed < 64) {
			quantize_speed++;
			ui_update = TRUE;
		    }
		    break;

		case 'd':
		    if (pattern_density > 0) {
			pattern_density--;
			ui_update = TRUE;
		    }
		    break;

		case 'D':
		    if (pattern_density < 64) {
			pattern_density++;
			ui_update = TRUE;
		    }
		    break;

		case 'm':
		    if (mutation_rate > 0) {
			mutation_rate--;
			ui_update = TRUE;
		    }
		    break;

		case 'M':
		    if (mutation_rate < 64) {
			mutation_rate++;
			ui_update = TRUE;
		    }
		    break;

		case 8:     /* backspace */
		case 127:   /* delete */
		    destroy_all_patterns();
		    break;

		case 32:    /* space */
		    t = clock();
		    all_notes_off();
		    clear_screen();
		    screen_out("Paused: press a key to continue");
		    readkey();
		    adjust_times((long)clock() - t);
		    bend = 0xFFFF;
		    ui_update = TRUE;
		    break;

		case 'v':
		case 'V':
		    view_list = the_mutators;
		    strcpy(view_title, "Mutator Probabilities");
		    ui_update = TRUE;
		    clear_screen();
		    bend = 0xFFFF;
		    break;

		case 27:    /* esc */
		    *finish = TRUE;
		    break;
	    }
	}
    }

    if (recording != is_recording()) {
	recording = !recording;
	ui_update = TRUE;
    }

    if (ui_update) {
	update_screen();
	ui_update = FALSE;
    }

    if (bend != pitch_bend) {
	bend = pitch_bend;
	draw_feedback((long)bend);
    }
}

